<?php

namespace Aimeos\MW\Jsb2\Standard;


/**
 * Test class for \Aimeos\MW\Jsb2\Standard.
 *
 * @copyright Metaways Infosystems GmbH, 2011
 * @license LGPLv3, http://opensource.org/licenses/LGPL-3.0
 * @copyright Aimeos (aimeos.org), 2015
 */
class StandardTest extends \PHPUnit_Framework_TestCase
{
	private $object;
	private $manifestPath;
	private $deployPath;


	protected function setUp()
	{
		$ds = DIRECTORY_SEPARATOR;
		$this->manifestPath = __DIR__ . $ds . 'manifests' . $ds;
		$this->object = new \Aimeos\MW\Jsb2\Standard( $this->manifestPath . 'manifest.jsb2' );
	}


	public function testConstructNoIncludeFilesExceptions()
	{
		$this->setExpectedException( '\\Aimeos\\MW\\Jsb2\\Exception' );
		$this->object = new \Aimeos\MW\Jsb2\Standard( $this->manifestPath . 'manifest_invalid_fileinclude.jsb2' );
	}


	public function testConstructNoPackageExceptions()
	{
		$this->setExpectedException( '\\Aimeos\\MW\\Jsb2\\Exception' );
		$this->object = new \Aimeos\MW\Jsb2\Standard( $this->manifestPath . 'manifest_invalid_package.jsb2' );
	}


	public function testConstructInvalidPackageContentExceptions()
	{
		$this->setExpectedException( '\\Aimeos\\MW\\Jsb2\\Exception' );
		$this->object = new \Aimeos\MW\Jsb2\Standard( $this->manifestPath . 'manifest_invalid_package_content.jsb2' );
	}


	public function testConstructNotJSONExceptions()
	{
		$this->setExpectedException( '\\Aimeos\\MW\\Jsb2\\Exception' );
		$this->object = new \Aimeos\MW\Jsb2\Standard( $this->manifestPath . 'manifest_no_json.jsb2' );
	}


	public function testConstructFileNotExistingExceptions()
	{
		$this->setExpectedException( '\\Aimeos\\MW\\Jsb2\\Exception' );
		$this->object = new \Aimeos\MW\Jsb2\Standard( $this->manifestPath . 'manifest_not_existing.jsb2' );
	}


	public function testGetFiles()
	{
		$files = $this->object->getFiles( 'js' );

		$this->assertEquals( 1, count( $files ) );
		$this->assertContains( 'test.js', $files[0] );
	}


	public function testGetHTML()
	{
		$html = '<script type="text/javascript" src="/./../%1$s"></script>';
		$mtime = filemtime( __DIR__ . DIRECTORY_SEPARATOR . 'test.js' );

		$this->assertEquals( sprintf( $html, 'test.js?v=' . $mtime ), trim( $this->object->getHTML( 'js' ) ) );
	}


	public function testGetUrls()
	{
		$urls = $this->object->getUrls( 'js' );

		$this->assertEquals( 1, count( $urls ) );
		$this->assertContains( '/test.js', $urls[0] );
	}


	public function testGetUrlsFilemtimeException()
	{
		$object = new \Aimeos\MW\Jsb2\Standard( $this->manifestPath . 'manifest_filemtime_exception.jsb2' );

		$this->setExpectedException( '\\Aimeos\\MW\\Jsb2\\Exception' );
		$object->getHTML( 'js' );
	}
}